package com.ym.mec.teacher.controller;

import com.alibaba.fastjson.JSON;
import com.dayaedu.cbs.openfeign.wrapper.music.CbsMusicSheetWrapper;
import com.microsvc.toolkit.common.response.template.R;
import com.ym.mec.biz.dal.dao.TeacherDao;
import com.ym.mec.biz.dal.entity.Instrument;
import com.ym.mec.biz.dal.entity.SysMusicScore;
import com.ym.mec.biz.dal.entity.Teacher;
import com.ym.mec.biz.dal.enums.ClientTypeEnum;
import com.ym.mec.biz.dal.page.SysExamSongQueryInfo;
import com.ym.mec.biz.dal.wrapper.InstrumentWrapper;
import com.ym.mec.biz.dal.wrapper.MusicSheetWrapper;
import com.ym.mec.biz.service.InstrumentService;
import com.ym.mec.biz.service.SysMusicScoreService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.vo.MusicSheetVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Slf4j
@Validated
@RestController
@RequestMapping("${app-config.url.teacher:}/musicSheet")
@Api(tags = "曲谱表")
public class MusicSheetController {

    @Autowired
    private SysMusicScoreService musicSheetService;

    @Autowired
    private InstrumentService instrumentService;

    @Resource
    private TeacherDao teacherDao;
    @Resource
    private SysUserService sysUserService;

    /**
     * 查询单条
     *
     * @param id 详情ID
     * @return R<MusicSheetVo.MusicSheet>
     */
    @ApiOperation(value = "查询内容平台数据")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "id", value = "id", dataType = "long")
    })
    @GetMapping("/cbsDetail/{id}")
    public R<MusicSheetVo.MusicSheetDetail> cbsDetail(@PathVariable("id") Integer id) {
        CbsMusicSheetWrapper.MusicSheet musicSheet = musicSheetService.cbsDetail(id);
        musicSheet.setBizId(Long.valueOf(id));

        MusicSheetVo.MusicSheetDetail musicSheetDetail = JSON.parseObject(JSON.toJSONString(musicSheet), MusicSheetVo.MusicSheetDetail.class);
        SysMusicScore detail = musicSheetService.get(id);
        if (detail == null) {
            musicSheetDetail.setBizMusicCategoryId(null);
        } else {
            //musicSheetDetail.setPaymentType(detail.get());
            musicSheetDetail.setBizMusicCategoryId(detail.getCategoriesId() == null ? null : detail.getCategoriesId().longValue());
        }

        // 设置乐器信息
        if (StringUtils.isNotBlank(musicSheetDetail.getMusicalInstrumentIds())) {
            List<Integer> instrumentIds = Arrays.stream(musicSheetDetail.getMusicalInstrumentIds().split(","))
                .map(Integer::parseInt).collect(Collectors.toList());

            if (CollectionUtils.isNotEmpty(instrumentIds) && CollectionUtils.isNotEmpty(musicSheetDetail.getMusicalInstruments())) {
                Map<Integer, InstrumentWrapper.Instrument> instrumentMap = instrumentService.getMapByIds(instrumentIds);
                for (CbsMusicSheetWrapper.MusicalInstrument musicalInstrument : musicSheetDetail.getMusicalInstruments()) {
                    InstrumentWrapper.Instrument instrument = instrumentMap.get(musicalInstrument.getId());
                    if (instrument != null) {
                        musicalInstrument.setOrientation(instrument.getOrientation());
                    }

                }
            }
        }
        return R.from(musicSheetDetail);
    }


    @ApiOperation(value = "查询分页", notes = "曲谱表- 传入 MusicSheetVo.MusicSheetQuery")
    @PostMapping("/page")
    public R<PageInfo<MusicSheetWrapper.MusicSheet>> page(@RequestBody MusicSheetVo.MusicSheetQuery query) {

        //判断用户是否可以访问云练习
        Integer userId = sysUserService.getUserId();
        Teacher teacher = teacherDao.get(userId);
        if (teacher == null) {
            throw new BizException("老师信息不存在");
        }
        //是否限制用户使用云教练
        if(teacher.getCoursewareLimit()){
            throw new BizException("您没有权限使用云练习");
        }
        SysExamSongQueryInfo queryInfo = new SysExamSongQueryInfo();

        if(query.getMusicSheetCategoriesId() != null) {
            queryInfo.setCategoriesId(query.getMusicSheetCategoriesId().toString());
        }
        if (query.getMusicalInstrumentId() != null) {
            // 乐器转换为声部
            Instrument instrument = instrumentService.getById(query.getMusicalInstrumentId());
            if (instrument != null) {
                queryInfo.setSubjectId(instrument.getSubjectId());
            }
        }
        queryInfo.setType("COMMON");
        queryInfo.setClientFlag(true);
        queryInfo.setClientType(ClientTypeEnum.SMART_PRACTICE);
        PageInfo<SysMusicScore> sysMusicScorePageInfo1 = musicSheetService.queryMusicScorePageInfo(queryInfo);
        PageInfo<MusicSheetWrapper.MusicSheet> convert = sysMusicScorePageInfo1.convert(o -> {
            MusicSheetWrapper.MusicSheet musicSheet = new MusicSheetWrapper.MusicSheet();
            musicSheet.setId(o.getId().toString());
            musicSheet.setMusicSheetCategoriesId(o.getCategoriesId());
            musicSheet.setMusicSheetName(o.getName());
            musicSheet.setTitleImg(o.getTitleImg());
            musicSheet.setComposer(o.getComposer());
            musicSheet.setCbsMusicSheetId(o.getCbsMusicSheetId());
            musicSheet.setUsedNum(0L);
            musicSheet.setPaymentType(StringUtils.isNotBlank(o.getRankIds()) && o.getRankIds().contains("1") ? "VIP" : "FREE");
            musicSheet.setAudioPlayTypes(o.getAudioPlayTypes());

            return musicSheet;
        });


        return R.from(convert);
    }
}
